<?php
// $Id: qtc.theme.inc,v 1.1.2.2 2010/07/23 12:13:44 quiptime Exp $

/**
 * @file
 * Theme function for qtc.module
 */

/**
 * Implementation of hook_theme().
 */
function qtc_theme($existing, $type, $theme, $path) {
  $hooks['users_block'] = array(
    'arguments' => array('items')
  );
  $hooks['friends_block'] = array(
    'arguments' => array('items')
  );
  $hooks['toggler_block'] = array(
    'arguments' => array()
  );
  $hooks['qtc_history_popup'] = array(
    'arguments' => array('content' => NULL),
    'template' => 'qtc-history-popup',
  );
  $hooks['qtc_bar']  = array(
    'arguments' => array(),
  );
  
  return $hooks;
}

/**
 * Theme function to display the QTChat bar.
 *
 * @return string
 *   The rendered chat bar.
 */
function theme_qtc_bar() {
  global $user;
  
  if (!variable_get('qtc_chat_bar', 0) || $user->uid == 0 || !user_access('access qtchat')) {
    return '';
  }

  $friend_module = variable_get('qtc_chat_friend_module', FALSE);

  // Check if in the settings an views display enabled.
  if (variable_get('qtc_chat_bar_counter', FALSE)) {
    $view = variable_get('qtc_chat_bar_counter', FALSE);
    $to_embed = explode('|', $view);
    $counter = views_embed_view($to_embed[0], $to_embed[1]);
  }
  elseif (!variable_get('qtc_chat_bar_counter', FALSE) && $friend_module) {
    $friends_data = qtc_chat_user_online_data();
    $counter = $friends_data['counter'];
  }

  $user_online_status = qtc_chat_user_online($user->uid);
  if ($user_online_status) {
    $user_online_status_class = 'qtc-user-online';
  }
  else {
    $user_online_status_class = 'qtc-user-offline';
  }

  // Options. Part of flying chat bar content.
  $options = '';
  $get_options = _qtc_chat_bar_options();
  if (count($get_options)) {
    $options = '<div id="qtc-barfly-options" style="display: none;">';
    $options .= '<div id="qtc-barfly-options-wrapper">';
    $options .= '<div id="qtc-barfly-options-base">';
    $options .= implode($get_options);
    $options .= '</div>';
    $options .= '</div>';
    $options .= '</div>';
  }

  // Build flying chat bar content.
  $barfly = '<div id="qtc-barfly" style="display: none;">'; // barfly
  
  $barfly .= '<div id="qtc-barfly-title">'; // barffly-title
  $barfly .= '<div id="qtc-barfly-title-wrapper" class="clear-block">';
  $barfly .= '<div class="qtc-barfly-header-title">'. 'Chat' .'</div>';
  $barfly .= '<div class="qtc-barfly-apps-hide"> - </div>';
  $barfly .= '</div>';
  $barfly .= '</div>';
  
  $barfly .= '<div id="qtc-barfly-header" class="clear-block">'; // barfly-header
  $barfly .= '<div class="clear-block">';
  $barfly .= '<div class="qtc-barfly-lists" style="display: inline; float: left;">'. t('Friend lists') .'</div>';
  $barfly .= '<div class="qtc-barfly-tools" style="display: inline; float: right;">'. t('Options') .'</div>';
  $barfly .= '</div>';
  $barfly .= $options;
  $barfly .= '</div>';

  $barfly .= '<div id="qtc-barfly-body" style="height: auto;" class="clear-block">'; // barfly-body
  $barfly .= '<div id="qtc-barfly-body-content" class="clear-block">';

  // Additional content. Upcoming features.
  if ($additional_content) {
    $barfly .= '<div id="qtc-barfly-additonal-content" class="clear-block">';
    $barfly .= '';
    $barfly .= '</div>';
  }
  
  // Online friends list.
  if ($friend_module) {
    $barfly .= '<div id="qtc-barfly-friends-list" class="clear-block">';
    $barfly .= _qtc_chat_bar_friends();
    $barfly .= '</div>';
  }

  $barfly .= '<div class="qtc-barfly-friends-list-message" style="display: none;">'. t('Loading data ...') .'</div>';

  $barfly .= '</div>';
  $barfly .= '</div>'; // barfly-body
  $barfly .= '</div>'; // barfly

  // Build chat bar content.
  $bar = '<div id="qtc-bar-onoffline" style="cursor: pointer;">';
  $bar .= '<div id="qtc-bar-onoffline-wrapper" class="clear-block">';
  
  // This div contains (CSS) the user online/offline icon.
  $bar .= '<div id="qtc-bar-online-indicator" class="'. $user_online_status_class .'" style="display: inline; float: left"> </div>';

  $bar .= '<div class="qtc-bar-user-online-text" style="display: inline; float: left">'. t('Chat').': </div>';

  // This div contains (CSS) the user online counter.
  $bar .= '<div class="qtc-bar-user-online-counter" style="display: inline; float: left">'. $counter .'</div>';

  $bar .= '</div>';
  $bar .= '</div>';

  // Add flying content.
  $bar .= $barfly;

  $output = '<div id="qtc-bar-container">';
  $output .= '<div id="qtc-bar-sub">';
  $output .= '<div id="qtc-bar">';
  $output .= $bar;
  $output .= '</div>';
  $output .= '</div>';
  $output .= '</div>';

  return $output;
}

/**
 * Provides the chat bar options itself.
 *
 * @return array
 *   The array contains the available rendered options.
 *
 * @see theme_qtc_bar()
 */
function _qtc_chat_bar_options() {
  global $user;

  $options = array();
  
  $status_variant = variable_get('qtc_online_status_variant', FALSE);

  // Provides the function to switch the online status.
  if ($status_variant) {
    $user_online = qtc_chat_user_online($user->uid);

    // Status parameter
    // 0 | Go offline
    // 1 | Go online
    if ($user_online == 1) {
      $online_toggle = 'block';
      $offline_toggle = 'none';
    }
    else {
      $online_toggle = 'none';
      $offline_toggle = 'block';
    }
    // Toggle the online/offline link.
    $output = '<a id="qtc-barfly-online-indicator-on" title="'. t('Go offline') .'" style="display: '. $online_toggle .';" class="qtc-go-offline" onclick="javascript:goOffOnline(\'0\', \''. $user->uid .'\')" href="javascript:void(0)">'. t('Go offline') .'</a>';
    $output .= '<a id="qtc-barfly-online-indicator-off" title="' .t('Go online') .'" style="display: '. $offline_toggle .';" class="qtc-go-online" onclick="javascript:goOffOnline(\'1\', \''. $user->uid .'\')" href="javascript:void(0)">'. t('Go online') .'</a>';
  
    $options['online_status'] = '<div class="qtc-barfly-action-status-toggle">'. $output .'</div>';
  }

  // Other options coming soon.

  return $options;
}

/**
 * Provides the chat bar friends list.
 *
 * @see theme_qtc_bar()
 */
function _qtc_chat_bar_friends() {
  $output = '';

  $friend_module = variable_get('qtc_chat_friend_module', 0);

  if ($friend_module) {
    $output = '<div id="qtc-barfly-friends-list-wrapper">';
    $output .= '<span style="margin: 8px;">'. t('Loading data ...') .'</span>';
    $output .= '</div>';
  }

  return $output;
}

/**
 * List users in block.
 */
function theme_users_block($items, $title = '') {
  global $user;
  
  $output = '';

  $output .= '<div class="item-list">'."\n";
  $output .= '<h3>' . $title . '</h3>'."\n";
  $output .= '<ul>'."\n";

  if (empty($items)) {
    $output .= '<li id="qtc-user-empty">'. t('There are no active users.') .'</li>'."\n";
  }
  else {
    foreach ($items as $i) {
      $name_toggle = qtc_user_name($i->name, $i->realname);
      if ($i->name != $user->name) {
        $output .= '<li id="qtc-user">';
        $output .= '<a class="qtc-chat-invite" href="javascript:void(0)" onclick="javascript:chatWith(\''. addslashes($i->uid) .'\', \''. addslashes($i->name) .'\', \''. $name_toggle .'\')">'. $name_toggle .'</a>';
        $output .= '</li>'."\n";
      }
      else {
        $output .= '<li id="qtc-user">'. $name_toggle .'</li>'."\n";
      }
    }
  }

  $output .= '</ul>'."\n";
  $output .= '</div>'."\n";

  return $output;
}

/**
 * List online friends in block.
 *
 *   If no friend module enabled this block is no available.
 */
function theme_friends_block($items, $title = FALSE) {
  global $user;

  $chat_invite_title = variable_get('qtc_chat_invite_title', 1);
  $chat_bar = variable_get('qtc_chat_bar', 0);

  $invite_title = t('Chat with user');
  if ($chat_invite_title == 2) {
    $invite_title = t('Invite to chat');
  }
  if ($chat_invite_title == 3) {
    $invite_title = t('Chat');
  }

  $output = '';

  $output .= '<div class="item-list">'."\n";
  if ($title) {
    $output .= '<h3>' . $title . '</h3>'."\n";
  }

  if (!$chat_bar) {
    // Don't change the function parameter.
    $output .= qtc_onoffline_toggler('qtc-friends-block');
  }

  $output .= '<div id="qtc-friends-block-list-wrapper">'."\n";

  if (empty($items)) {
    $output .= '<span id="qtc-friends-empty">'. t('There are no online friends.') .'</span>'."\n";
  }
  else {
    foreach ($items as $i) {
      $name_toggle = qtc_user_name($i['name'], $i['realname']);
      if ($i['name'] != $user->name) {
        $output .= '<a class="qtc-chat-invite" href="javascript:void(0)" style="display: block;" title="'. $invite_title .'" onclick="javascript:chatWith(\''. addslashes($i['uid']) .'\', \''. addslashes($i['name']) .'\', \''. $name_toggle .'\')">'. $name_toggle .'</a>';
      }
    }
  }

  $output .= '</div>'."\n";
  $output .= '</div>'."\n";

  return $output;
}

/**
 * Provides an block to allow users to switch their online status.
 */
function theme_toggler_block($title = FALSE) {
  global $user;

  $output = '';

  if ($title) {
    $output .= '<h3>' . $title . '</h3>'."\n";
  }

  // Don't change the function parameter.
  $output .= qtc_onoffline_toggler('qtc-toggler-block');

  return $output;
}

/**
 * Provides an online/offline toggle to allow an user to switch their online status.
 * 
 * @param string $id
 *   An CSS id part.
 *   This CSS id part is usable in the corresponding jQuery functionality.
 * @return string
 *   The rendered toggle.
 *
 * @see theme_friends_block()
 */
function qtc_onoffline_toggler($id) {
  global $user;
  
  $user_online = qtc_chat_user_online($user->uid);
  $output .= '<div id="'. $id .'-onoffline-wrapper" class="clear-block">';

  if ($user_online == TRUE) {
    $toggle_on = 'display: block;';
    $toggle_off = 'display: none;';
  }
  else {
    $toggle_on = 'display: none;';
    $toggle_off = 'display: block;';
  }

  $output .= '<div id="'. $id .'-toggler-on" style="'. $toggle_on .'">';
  $output .= '<div class="qtc-toggler-status-message">'. t('You are online') .'</div>';
  $output .= '<a id="'. $id .'-online-indicator-on" class="'. $id .'-go-online" href="javascript:void(0)" onclick="javascript:goOffOnline(\'0\', \''. $user->uid .'\')" title="'. t('Go offline') .'">';
  $output .= '<div id="qtc-toggler-online-indicator" class="'. $id .'-user-online"> </div>';
  $output .= '</a>';
  $output .= '</div>';

  $output .= '<div id="'. $id .'-toggler-off" style="'. $toggle_off .'">';
  $output .= '<div class="qtc-toggler-status-message">'. t('You are offline') .'</div>';
  $output .= '<a id="'. $id .'-online-indicator-off" class="'. $id .'-go-online" href="javascript:void(0)" onclick="javascript:goOffOnline(\'1\', \''. $user->uid .'\')" title="'. t('Go online') .'">';
  $output .= '<div id="qtc-toggler-online-indicator" class="'. $id .'-user-offline"> </div>';
  $output .= '</a>';
  $output .= '</div>';

  $output .= '</div>';

  return $output;
}

/**
 * Fill in a bunch of page variables for our specialized popup page.
 */
function template_preprocess_qtc_history_popup(&$variables) {
  global $theme;

  // Construct page title.
  if (drupal_get_title()) {
    $head_title = array(strip_tags(drupal_get_title()), variable_get('site_name', 'Drupal'));
  }
  else {
    $head_title = array(variable_get('site_name', 'Drupal'));
    if (variable_get('site_slogan', '')) {
      $head_title[] = variable_get('site_slogan', '');
    }
  }

  drupal_add_css(drupal_get_path('module', 'qtc') . '/css/history-popup.css');

  $variables['head_title']        = implode(' | ', $head_title);
  $variables['base_path']         = base_path();
  $variables['front_page']        = url();
  $variables['head']              = drupal_get_html_head();
  $variables['language']          = $GLOBALS['language'];
  $variables['language']->dir     = $GLOBALS['language']->direction ? 'rtl' : 'ltr';
  $variables['css']               = drupal_add_css();
  $css = drupal_add_css();

  // Remove theme css.
  foreach ($css as $media => $types) {
    if (isset($css[$media]['theme'])) {
      $css[$media]['theme'] = array();
      //$css[$media]['module'] = array();
    }
  }

  $variables['styles']            = drupal_get_css($css);
  //$variables['scripts']           = drupal_get_js();
  $variables['title']             = drupal_get_title();
  // Closure should be filled last.
  $variables['closure']           = theme('closure');
}
