<?php
// $Id: qtc.admin.inc,v 1.1.2.5 2010/07/23 12:13:43 quiptime Exp $

/**
 * @file
 * Administer QTChat.
 */

/**
 * Administer QTChat.
 *
 *   - Global settings
 *   - Chat settings
 *
 * @todo
 *
 *   Insert clear cache functionality to make available changes of the qtc_online_status_variant variable!
 */
function qtc_admin_form() {
  $path = drupal_get_path('module', 'qtc');
  drupal_add_js($path . '/js/qtc_admin.js');

  $destination = drupal_get_destination();
  $messages_count = db_result(db_query('SELECT COUNT(zid) as count FROM {qtc}'));
  $form = array();

  $form['global'] = array(
    '#type'        => 'fieldset',
    '#title'       => t('Global settings'),
    '#collapsible' => FALSE,
    '#collapsed'   => FALSE,
  );
  $form['global']['messages'] = array(
    '#type'        => 'fieldset',
    '#title'       => t('Messages'),
    '#collapsible' => FALSE,
    '#collapsed'   => FALSE
  );
  $description_m = t('Age for chat messages to be deleted. The functionality to delete old chat messages requires a cron task.') .'<br />';
  $description_m .= '<strong>'. t('Important') .':</strong> ';
  $description_m .= t('Make sure that is not shorter %messages than %history.', array('%messages' => t('Delete messages'), '%history' => t('Delete histories')));
  $form['global']['messages']['qtc_old_messages'] = array(
    '#type'          => 'select',
    '#title'         => t('Delete messages'),
    '#description'   => $description_m,
    '#options'       => _qtc_admin_time_options(),
    '#default_value' => variable_get('qtc_old_messages', 86400),
  );

  $form['global']['messages']['messages_info'] = array(
    '#type'  => 'markup',
    '#value' => t('The database contains <strong>!count</strong> chat messages.', array('!count' => $messages_count)),
  );
  $form['global']['messages']['cron'] = array(
    '#type'        => 'fieldset',
    '#title'       => t('Cron'),
    '#collapsible' => FALSE,
    '#collapsed'   => FALSE
  );
  
  $value = t('You can <a href="@cron">run cron manually</a>.', array(
    '@cron' => url('admin/reports/status/run-cron', array('query' => $destination)))
  );
  $form['global']['messages']['cron']['cron_run'] = array(
    '#type'  => 'markup',
    '#value' => $value,
  );
  $form['global']['messages']['qtc_history'] = array(
    '#type'          => 'select',
    '#title'         => t('Delete histories'),
    '#description'   => t('Age for chat histories to be deleted. If a chat window is closed then deleted the chat histories according to this timing - this refers to the user who has chatted.'),
    '#options'       => _qtc_admin_time_options(),
    '#default_value' => variable_get('qtc_history', 43200),
  );

  // Online status settings.
  $form['online_status'] = array(
    '#type'        => 'fieldset',
    '#title'       => t('Online status'),
    '#collapsible' => FALSE,
    '#collapsed'   => FALSE,
  );
  
  $description_status_variant = t('If <em>Drupals default</em> selected: Users can not using the online/offline functionality.') .'<br />';
  $description_status_variant .= t('If <em>Accept users status</em> selected: Users can set your online status.');
  $form['online_status']['qtc_online_status_variant'] = array(
    '#type'          => 'select',
    '#title'         => t('Online status variant'),
    '#description'   => $description_status_variant,
    '#options'       => array(1 => t('Yes, accept users status'), 0 => t('No, use Drupals default')),
    '#default_value' => variable_get('qtc_online_status_variant', FALSE),
  );
  $form['online_status']['qtc_online_status_variant_old'] = array(
    '#type' => 'value',
    '#value' => variable_get('qtc_online_status_variant', FALSE),
  );

  $description_status_time = t('Choose the time after which a user status is offline.') .' '; 
  $description_status_time .= t('Note: The corresponding time of the %user-stats module and Drupals <em>User online block</em> is 15 minutes.', array('%user-stats' => t('User Stats'))) .'<br />';
  $description_status_time .= t('If selected the option <em>Accept users status</em> from the setting %status-variant, then has the setting of the user priority.', array('%status-variant' => t('Online status variant')));
  $form['online_status']['qtc_online_status_time'] = array(
    '#type'          => 'select',
    '#title'         => t('Online status time'),
    '#description'   => $description_status_time,
    '#options'       => _qtc_online_time_options(),
    '#default_value' => variable_get('qtc_online_status_time', 900),
  );

  // Chat settings.
  $form['chat'] = array(
    '#type'        => 'fieldset',
    '#title'       => t('Chat settings'),
    '#collapsible' => FALSE,
    '#collapsed'   => FALSE,
  );
  $form['chat']['qtc_chat_bar'] = array(
    '#type'          => 'select',
    '#title'         => t('Chat bar'),
    '#description'   => t('Choose whether to use the chat bar.'),
    '#options'       => array(0 => t('No'), 1 => t('Yes')),
    '#default_value' => variable_get('qtc_chat_bar', 0),
  );
  $bar_counter = array(0 => t('Don\'t use an view'));
  $options_bar_counter = $bar_counter + qtc_get_views('users');
  $description_user_counter = t('Choose an view display to get an counter result of online users. The counter should count the online users or the online friends of the current user.') .' ';
  $description_user_counter .= t('The counter is used mainly in the chat bar.');
  $form['chat']['qtc_chat_user_counter'] = array(
    '#type'          => 'select',
    '#title'         => t('User counter'),
    '#description'   => $description_user_counter,
    '#options'       => $options_bar_counter,
    '#default_value' => variable_get('qtc_chat_user_counter', FALSE),
  );

  $form['chat']['modules'] = array(
    '#type'        => 'fieldset',
    '#title'       => t('Friend modules'),
    '#collapsible' => FALSE,
    '#collapsed'   => FALSE,
  );
  $friend_modules = _qtc_check_friend_modules();
  $friend_module_options = array(0 => t('Do not use friend modules')) + $friend_modules;
  if (count($friend_modules)) {
    $form['chat']['modules']['qtc_chat_friend_module'] = array(
      '#type'          => 'select',
      '#title'         => t('Friend modules'),
      '#description'   => t('Choose a friend module.'),
      '#options'       => $friend_module_options,
      '#default_value' => variable_get('qtc_chat_friend_module', 0),
      '#suffix' => '<div id="friend_module-select" class="warning" style="display: none;">'. t('Changes made in the option %modules must be saved with %submit it can be configured the related options.', array('%modules' => t('Friend modules'),'%submit' => t('Save configuration'))) .'</div>',
    );
  }
  else {
    // No friend modules available.
    $upported = qtc_friend_modules();
    $no_friend_module = theme_item_list($upported, t('Supported friend modules'));
    $form['chat']['modules']['no_friend_module'] = array(
      '#type' => 'markup',
      '#value' => $no_friend_module,
    );
  }

  if (variable_get('qtc_chat_friend_module', FALSE)) {
    if (variable_get('qtc_chat_friend_module', FALSE) == 'flag_friend') {
      $result = db_query("SELECT fid, name, title FROM {flags} WHERE content_type = 'user' AND global = 0");
      $ff_flags = array();
      while ($type = db_fetch_object($result)) {
        $ff_flags[$type->fid] = array('name' => $type->name, 'title' => $type->title);
        $ff_options[$type->fid] = $type->title;
      }
      
      $ff_multiple = count($ff_options);
      $form['chat']['modules']['qtc_chat_friend_module_flag_friend'] = array(
        '#type'          => 'select',
        '#title'         => t('Friend flags'),
        '#description'   => t('Choose one or more friend flags. At least one flag is required for this to work the friends online counter and the friends list function.'),
        '#options'       => $ff_options,
        '#multiple'      => TRUE,
        '#size'          => $ff_multiple ? min(3, count($ff_options)) : 0,
        '#default_value' => variable_get('qtc_chat_friend_module_flag_friend', array()),
      );
    }
    elseif (variable_get('qtc_chat_friend_module', FALSE) == 'user_relationships_api') {
      $ur_flags = user_relationships_types_load();
      if (count($ur_flags)) {
        foreach ($ur_flags as $key => $rel) {
          $ur_options[$rel->rtid] = $rel->plural_name;
        }

        $ur_multiple = count($ur_options);
        $form['chat']['modules']['qtc_chat_friend_module_user_relationships'] = array(
          '#type'          => 'select',
          '#title'         => t('Relationships'),
          '#description'   => t('Choose one ore more user relationship types. At least one relationship is required for this to work the friends online counter and the friends list function.'),
          '#options'       => $ur_options,
          '#multiple'      => TRUE,
          '#size'          => $ur_multiple ? min(3, count($ur_options)) : 0,
          '#default_value' => variable_get('qtc_chat_friend_module_user_relationships', array()),
        );
      }
    }

    $form['chat']['qtc_chat_invite_title'] = array(
      '#type'          => 'select',
      '#title'         => t('Invite title'),
      '#description'   => t('Select the text that is used to chat with a user. Example for use: Mouse over text of user links.'),
      '#options'       => array(
                            1 => t('Chat with user'),
                            2 => t('Invite to chat'),
                            3 => t('Chat'),
                          ),
      '#default_value' => variable_get('qtc_chat_invite_title', 1),
    );
  }

  return system_settings_form($form);
}

/**
 * Administer the QTChat heartbeat settings.
 */
function qtc_admin_form_heartbeat() {

  // Heatbeat settings.
  $description_heartbeat = t('The heartbeat time is the frequency with which the clients (browsers) connects to the server.') .'<br />';
  $description_heartbeat .= t('Be very careful with this settings, as it can quickly eat your server resources.');
  $form['heartbeat']= array(
    '#type'        => 'fieldset',
    '#title'       => t('Heartbeat'),
    '#description' => $description_heartbeat,
    '#collapsible' => FALSE,
    '#collapsed'   => FALSE,
  );

  $form['heartbeat']['qtc_heartbeat_min']= array(
    '#type'          => 'select',
    '#title'         => t('Time min'),
    '#description'   => t('Minimum heartbeat time.'),
    '#options'       => array(2000 => '2 '. t('seconds'), 4000 => '4 '. t('seconds'), 6000 => '6 '. t('seconds'), 8000 => '8 '. t('seconds'), 9990 => '10 '. t('seconds')),
    '#default_value' => variable_get('qtc_heartbeat_min', 6000),
  );
  $form['heartbeat']['qtc_heartbeat_max']= array(
    '#type'          => 'select',
    '#title'         => t('Time max'),
    '#description'   => t('Maximum heartbeat time.'),
    '#options'       => array(40000 => '40 '. t('seconds'), 30000 => '30 '. t('seconds'), 25000 => '25 '. t('seconds'), 20000 => '20 '. t('seconds')),
    '#default_value' => variable_get('qtc_heartbeat_max', 30000),
  );
  $description_count_1 = t('The number of client connects, after which time shall be extended for the first time.').'<br />';
  $description_count_1 .= t('After this number is extended the heartbeat time with the factor') .' 3.';
  $form['heartbeat']['qtc_heartbeat_count_1']= array(
    '#type'          => 'select',
    '#title'         => t('Time count one'),
    '#description'   => $description_count_1,
    '#options'       => array(15 => 15, 20 => 20, 25 => 25, 30 => 30),
    '#default_value' => variable_get('qtc_heartbeat_count_1', 15),
  );
  $description_count_2 = t('The number of client connects to increase the time count one, as result after which time shall be extended for the second time.').'<br />';
  $description_count_2 .= t('After this number is extended the heartbeat time with the factor') .' 4.';
  $form['heartbeat']['qtc_heartbeat_count_2']= array(
    '#type'          => 'select',
    '#title'         => t('Time count second'),
    '#description'   => $description_count_2,
    '#options'       => array(30 => 30, 25 => 25, 20 => 20, 15 => 15),
    '#default_value' => variable_get('qtc_heartbeat_count_2', 30),
  );

  $form['heartbeat']['userdata'] = array(
    '#type'        => 'fieldset',
    '#title'       => t('User data'),
    '#description' => t('Heartbeat time to get user data. They are user and friends lists, user and friends counter'),
    '#collapsible' => FALSE,
    '#collapsed'   => FALSE,
  );
  $form['heartbeat']['userdata']['qtc_heartbeat_userdata']= array(
    '#type'          => 'select',
    '#title'         => t('Data time'),
    '#description'   => t('By this time, updates the user data.'),
    '#options'       => array(60000 => '1 '. t('minute'), 90000 => '1,5 '. t('minutes'), 120000 => '2 '. t('minutes'), 150000 => '2,5 '. t('minutes'), 180000 => '3 '. t('minutes')),
    '#default_value' => variable_get('qtc_heartbeat_userdata', 120000),
  );

  return system_settings_form($form);
}

/**
 * Realize additional things from the QTChat global settings.
 */
function qtc_settings_admin_form_submit($form, $formstate) {
  // Make sure that the availability of the function to change the online status
  // is changed in the user account.
  if ($formstate['values']['qtc_online_status_variant_old'] != $formstate['values']['qtc_online_status_variant']) {
    menu_rebuild();
  }
}

/**
 * Help function to examine, what supported friend modules are enabled.
 *
 * @return array
 *   An associative array contains the available friend modules.
 */
function _qtc_check_friend_modules() {
  $friend_modules = array();

  foreach (qtc_friend_modules() as $module => $title) {
    if (module_exists($module)) {
      $friend_modules[$module] = t($title);
    }
    else {
      if (variable_get('qtc_chat_friend_module_'. $module, array())) {
        variable_del('qtc_chat_friend_module_'. $module);
      }
    }
  }

  // Functions to see:
  // flag_friend_get_friends($uid, $reset = NULL)
  // flag_friend_get_friend_count($uid, $reset = NULL)

  // user_relationship_defaults_load($reset = FALSE); user_relationship_defaults.module
  // user_relationship_default_load($param = array()); user_relationship_defaults.module
  // user_relationships_types_load($param = array(), $reset = NULL); user_relationships_api.api.inc
  // user_relationships_type_load($param = array(), $reset = NULL); user_relationships_api.api.inc

  return $friend_modules;
}

/**
 * Help function. Provides time options.
 *
 * @return array
 *   An associative array contains the time options.
 */
function _qtc_admin_time_options() {
  return array(
    3600 => '1 '. t('hour'),
    7200 => '2 '. t('hours'),
    10800 => '3 '. t('hours'),
    21600 => '6 '. t('hours'),
    43200 => '12 '. t('hours'),
    64800 => '18 '. t('hours'),
    86400 => '24 '. t('hours'),
    172800 => '2 '. t('days'),
    259200 => '3 '. t('days'),
    345600 => '4 '. t('days'),
    432000 => '5 '. t('days'),
    518400 => '6 '. t('days'),
    604800 => '7 '. t('days'),
  );
}
